/*
 * $Id$
 * Author: Karoly Bosa
 *
 * (C) 2010 by Research Institute for Symbolic Computation, Linz, Austria
 * All rights reserved. See COPYRIGHT in top-level directory.
 */

#include "GroupData.h"

GroupData::GroupData(int _nrOfProcs) {
	nrOfProcs = _nrOfProcs;
	indexOflastCheckedLC = 0;
	latencyLevel = 10000; //A BIG VALUE
	tmpLevel = 10000;     //A BIG VALUE
}

GroupData::~GroupData() {
	removeMappingTickets();
}

int GroupData::getNrOfProcs() {
	return nrOfProcs;
}

void GroupData::setIndexOfLatencyCluster(int index) {
	indexOflastCheckedLC = index;
}

int GroupData::getIndexOfLatencyCluster() {
	return indexOflastCheckedLC;
}

void GroupData::setTmpLevel(int _level) {
	tmpLevel = _level;
}

int GroupData::getTmpLevel() {
	return tmpLevel;
}

int GroupData::getLatencyLevel() {
	return latencyLevel;
}

void GroupData::createLoadTicket(int _nrOfProcs, HostData *host) {
	LoadTicket *ticket = new LoadTicket(_nrOfProcs, host);
	tmpMapping.push_back(ticket);
	host->increaseLoad(_nrOfProcs);
}

int GroupData::getNrOfLoadTickets() {
	return mapping.size();
}

int GroupData::getNrOfTmpTickets() {
	return tmpMapping.size();
}

LoadTicket *GroupData::getMappingTicketByIndex(unsigned int index) {
	if (mapping.empty() || index >= mapping.size()) return NULL;
	return mapping[index];
}

LoadTicket *GroupData::getTmpTicketByIndex(unsigned int index) {
	if (tmpMapping.empty() || index >= tmpMapping.size()) return NULL;
	return tmpMapping[index];
}

void GroupData::removeMappingTickets() {
	while (!mapping.empty()) {
		LoadTicket *ticket = mapping.back();
		mapping.pop_back();
		delete ticket;
	}
}

void GroupData::removeLastTmpTickets(int _nrOfProcs) {
	while (!tmpMapping.empty() && _nrOfProcs>0) {
		LoadTicket *ticket = tmpMapping.back();
		tmpMapping.pop_back();
		ticket->getHost()->decreaseLoad(ticket->getNrOfProcs());
		_nrOfProcs -= ticket->getNrOfProcs();
		delete ticket;
	}
}

void GroupData::copyTmpTickets() {
	if (!mapping.empty()) removeMappingTickets();
	latencyLevel = tmpLevel;
	vector<LoadTicket *>::iterator it;
	for (it = tmpMapping.begin(); it != tmpMapping.end(); it++) {
		LoadTicket *ticket = new LoadTicket((*it)->getNrOfProcs(), (*it)->getHost());
		mapping.push_back(ticket);
	}
}
